/*
 * host_platform.c
 *
 * Copyright(c) 1998 - 2009 Texas Instruments. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Texas Instruments nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "tidef.h"
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/sched.h>
#include <plat/tc.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <plat/wifi_tiwlan.h>

#include "host_platform.h"
#include "ioctl_init.h"
#include "WlanDrvIf.h"
#include "Device1273.h"


struct wifi_platform_data *wifi_control_data = NULL;
struct resource *wifi_irqres = NULL;
int wifi_probe_irq(struct platform_device *pdev);
int wifi_remove_irq(struct platform_device *pdev);
extern struct platform_device *gpdev;
void wifi_del_dev(void);

int wifi_probe_irq(struct platform_device *pdev)
{
	struct wifi_platform_data *wifi_ctrl = (struct wifi_platform_data *)(pdev->dev.platform_data);

	wifi_irqres = platform_get_resource_byname(pdev, IORESOURCE_IRQ, "device_wifi_irq");
	if (wifi_irqres) {
		printk(KERN_INFO "%s: got wlan irq %lu\n", __func__, (unsigned long)(wifi_irqres->start));
		printk(KERN_INFO "%s: got wlan irq trigger %s flag\n", __func__, 
			wifi_irqres->flags & IRQF_TRIGGER_FALLING ? "falling" : "unknown");
	}

	wifi_control_data = wifi_ctrl;
	return 0;
}
											
int wifi_remove_irq(struct platform_device *pdev)
{
	return 0;
}

static int unplug_device(int delay)
{
	if(wifi_control_data) {
		if(wifi_control_data->set_carddetect)
			wifi_control_data->set_carddetect(0);
		if(wifi_control_data->set_reset)
			wifi_control_data->set_reset(1);
		if(wifi_control_data->set_power)
			wifi_control_data->set_power(0);
	}

	/* let the mmc core finish enumeration + initialization before we continue */
	set_current_state(TASK_INTERRUPTIBLE);
	schedule_timeout(HZ);

    mdelay(delay);

	return 0;
}
int hPlatform_DevicePowerOff (void)
{
	return unplug_device(10);
}

/*--------------------------------------------------------------------------------------*/

/* Turn device power off according to a given delay */
int hPlatform_DevicePowerOffSetLongerDelay(void)
{
	return unplug_device(SDIO_ATTEMPT_LONGER_DELAY_LINUX);
}

/*--------------------------------------------------------------------------------------*/

/* Turn device power on */
int hPlatform_DevicePowerOn (void)
{

	if(!wifi_control_data || !wifi_control_data->set_power)
		return -1;
	
	wifi_control_data->set_power(1);
    /* New Power Up Sequence */
    mdelay(15);
	wifi_control_data->set_power(0);
    mdelay(1);
	wifi_control_data->set_power(1);
    /* Should not be changed, 50 msec cause failures */
    mdelay(70);

	if(wifi_control_data->set_reset)
		wifi_control_data->set_reset(0);
	if(wifi_control_data->set_carddetect)
		wifi_control_data->set_carddetect(1);

        /* let the mmc core finish enumeration + initialization before we continue */
        set_current_state(TASK_INTERRUPTIBLE);
        schedule_timeout(HZ);

    return 0;
}
#if 0
static void pad_config(unsigned long pad_addr, u32 andmask, u32 ormask)
{
     int val;
     u32 *addr;

     addr = (u32 *) ioremap(pad_addr, 4);
     if (!addr) {
         printk(KERN_ERR "OMAP3430_pad_config: ioremap failed with addr %lx\n", pad_addr);
         return;
     }

     val =  __raw_readl(addr);
     val &= andmask;
     val |= ormask;
     __raw_writel(val, addr);

     iounmap(addr);
}

static void wlan_muxcfg(void)
{

    pad_config(CM_ICLKEN1_CORE, 0xFFFFFFFF, MMC3_EN_MASK); /*I_CLK Enable MMC3*/
    pad_config(CM_FCLKEN1_CORE, 0xFFFFFFFF, MMC3_EN_MASK); /*F_CLK */

    pad_config(CONTROL_PADCONF_CAM_D1, 0xFFE0FFFF, 0x001C0000);

     /* choose gpio 162, pull up, activated */
     /* Setting MUX Mode 4 , Pull bits 3 */
     /* Should set (x is don't change):  xxxx xxxx xxxx xxxx xxxx xxxx xxx1 1100
    pad_config(CONTROL_PADCONF_MCBSP1_CLKX, 0xFFFFFFF0, 0x0000011C);

    Resetting the WLAN pad config register values to zero and setting it back to required values */
    pad_config(CONTROL_PADCONF_MCBSP1_CLKX, 0xFFFF0000, 0x00000000);
    pad_config(CONTROL_PADCONF_MCBSP1_CLKX, 0xFFFF0000, 0x0000C11C);

     /*
      *  set pull up on all SDIO lines
      *  Setting MUX Mode of 0, and pull bits to 3
      */
     /* set for mmc2_cmd - second half of the padconf register
      * Should set (x is don't change):  xxxx xxxx xxx1 1000 xxxx xxxx xxxx xxxx */
    pad_config(CONTROL_PADCONF_MMC3_CMD, 0xFFFFFFF0, 0x0000011B);
    pad_config(CONTROL_PADCONF_MMC3_CLK, 0xFFF0FFE0,0x001C011A);

     /* set for mmc3_dat0 and dat1 - both parts of the padconf register
      * Should set (x is don't change):  xxxx xxxx xxx1 1000 xxxx xxxx xxx1 1000 */
    pad_config(CONTROL_PADCONF_MMC3_DAT0, 0xFFF0FFF0, 0x011A011A);
    pad_config(CONTROL_PADCONF_MMC3_DAT2, 0xFFFFFFF0, 0x0000011A);
    pad_config(CONTROL_PADCONF_MMC3_DAT3, 0xFFF0FFFF, 0x011A0000);
    pad_config(CONTROL_PADCONF_MMC2_DAT4, 0xFFF0FFF0, 0x00180018);
    pad_config(CONTROL_PADCONF_MMC2_DAT6, 0xFFF0FFF0, 0x00180018);
}
#endif
/*int hPlatform_Wlan_Hardware_Init(void *tnet_drv)*/
int hPlatform_Wlan_Hardware_Init(void )
{
 #if 0
     wlan_muxcfg();
 #endif
    if(gpdev != NULL)
    {
        wifi_probe_irq(gpdev);
    }
	if (!wifi_irqres)
		return -1;

    /* Power on device to check whether HW is present
       In case tiwlan_sdio_probe does not called wlan script will take care
    */
	hPlatform_DevicePowerOn();

    return 0;
} /* hPlatform_Wlan_Hardware_Init() */



int hPlatform_initInterrupt(void *tnet_drv, void* handle_add ) 
{
	TWlanDrvIfObj *drv = tnet_drv;
    int rc;

	if (!wifi_irqres)
		return -1;

	drv->irq = wifi_irqres->start;
	drv->flags = wifi_irqres->flags & IRQF_TRIGGER_MASK;

	if (drv->irq == 0 || handle_add == NULL)
	{
	  print_err("hPlatform_initInterrupt() bad param drv->irq=%d handle_add=0x%x !!!\n",drv->irq,(int)handle_add);
	  return -EINVAL;
	}
        printk(" IRQ= %d FLAGS= %ld\n",drv->irq, drv->flags);  
	rc = request_irq(drv->irq, handle_add, drv->flags, drv->netdev->name, drv);
	if (rc) {
	    print_err("TIWLAN: Failed to register interrupt handler\n");
		return rc;
	}

	set_irq_wake(drv->irq, 1);
	return rc;

} /* hPlatform_initInterrupt() */

void hPlatform_freeInterrupt(void *tnet_drv)
{
	TWlanDrvIfObj *drv = tnet_drv;

	set_irq_wake(drv->irq, 0);
	free_irq(drv->irq, drv);
}

void *hPlatform_hwGetRegistersAddr(TI_HANDLE OsContext)
{
	return (void *)OS_API_REG_ADDR;
}

void *hPlatform_hwGetMemoryAddr(TI_HANDLE OsContext)
{
	return (void *)OS_API_MEM_ADDR;
}

void hPlatform_Wlan_Hardware_DeInit(void)
{
	wifi_del_dev();
}

